"""
Settings for feedaggregator project
"""
from django.conf import settings as django_settings

FEEDAGGREGATOR_PAGE_SIZE = getattr(django_settings, 'FEEDAGGREGATOR_PAGE_SIZE', 10)
FEEDAGGREGATOR_TITLE = getattr(django_settings, 'FEEDAGGREGATOR_TITLE', "Latest posts")
FEEDAGGREGATOR_SHOW_FULL = getattr(django_settings, 'FEEDAGGREGATOR_SHOW_FULL', False)
FEEDAGGREGATOR_EXCERPT_SIZE = getattr(django_settings, 'FEEDAGGREGATOR_EXCERPT_SIZE', 200)

# Own RSS feed
FEEDAGGREGATOR_RSS_SIZE = getattr(django_settings, 'FEEDAGGREGATOR_RSS_SIZE', 25)
FEEDAGGREGATOR_RSS_DESCRIPTION = getattr(django_settings, 'FEEDAGGREGATOR_RSS_DESCRIPTION', "")

# Tags
FEEDAGGREGATOR_TAGS_LOWERCASE = getattr(django_settings, 'FEEDAGGREGATOR_TAGS_LOWERCASE', True)



# -*- coding: utf-8 -*-
from feedaggregator.settings import FEEDAGGREGATOR_TITLE, FEEDAGGREGATOR_SHOW_FULL, FEEDAGGREGATOR_EXCERPT_SIZE


def feedaggregator_settings(request):
    return {
        "FEEDAGGREGATOR_TITLE": FEEDAGGREGATOR_TITLE,
        "FEEDAGGREGATOR_SHOW_FULL": FEEDAGGREGATOR_SHOW_FULL,
        "FEEDAGGREGATOR_EXCERPT_SIZE": FEEDAGGREGATOR_EXCERPT_SIZE
    }



# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url

from feedaggregator.views import ItemList, ItemDetail, FeedDetail, FeedList, TagDetail, TagList, TagCloud
from feedaggregator.feeds import LatestItemsFeed, AtomLatestItemsFeed


urlpatterns = patterns('feedaggregator.views',
    # Items
    url(r'^$', ItemList.as_view(), name="item_list"),
    url(r'^item/(?P<slug>[-\w]+)/$', ItemDetail.as_view(), name="item_detail"),

    # Feeds
    url(r'^feeds/$', FeedList.as_view(), name="feed_list"),
    url(r'^feeds/(?P<slug>[-\w]+)/$', FeedDetail.as_view(), name="feed_detail"),

    # Tags
    url(r'^tags/$', TagList.as_view(), name="tag_list"),
    url(r'^tag/(?P<tag>[-\w]+)/$', TagDetail.as_view(), name="tag_detail"),

    # Tag cloud
    url(r'^cloud/$', TagCloud.as_view(), name="tag_cloud"),

    # OPML
    url(r'^opml.xml$', 'export_opml', name="opml"),
)

# Feed's urls
urlpatterns += patterns('',
    url(r'^feed/rss/$', LatestItemsFeed(), name="rss_latest_items"),
    url(r'^feed/atom/$', AtomLatestItemsFeed(), name="atom_latest_items")
)



# -*- coding: utf-8 -*-
from django.shortcuts import render
from django.views.generic import DetailView, ListView, TemplateView
from django.contrib.sites.shortcuts import get_current_site

from taggit.models import Tag

from feedaggregator.models import Feed, Item
from feedaggregator.settings import FEEDAGGREGATOR_PAGE_SIZE


class FeedList(ListView):
    """
    List all the active feeds
    """
    model = Feed
    context_object_name = "feed_list"
    paginate_by = FEEDAGGREGATOR_PAGE_SIZE

    def get_queryset(self):
        return Feed.active.all()


class FeedDetail(ListView):
    """
    Show an feed information.
    Used a listview instead of a detailview to use pagination easily
    """
    model = Item
    context_object_name = "item_list"
    template_name = "feedaggregator/feed_detail.html"
    paginate_by = FEEDAGGREGATOR_PAGE_SIZE

    def get_context_data(self, **kwargs):
        context = super(FeedDetail, self).get_context_data(**kwargs)
        context['feed'] = Feed.objects.get(slug=self.kwargs['slug'])
        return context

    def get_queryset(self):
        return Item.active.filter(feed__slug=self.kwargs['slug'])


class ItemList(ListView):
    """
    List of all the active items
    """
    model = Item
    context_object_name = "item_list"
    paginate_by = FEEDAGGREGATOR_PAGE_SIZE

    def get_queryset(self):
        return Item.active.all()


class ItemDetail(DetailView):
    """
    Show an individual item
    """
    model = Item
    context_object_name = "item"


class TagList(ListView):
    model = Tag
    context_object_name = "tag_list"
    template_name = "feedaggregator/tag_list.html"
    paginate_by = FEEDAGGREGATOR_PAGE_SIZE


class TagDetail(ListView):
    """
    Show all the items that have that tag
    """
    model = Item
    context_object_name = "item_list"
    template_name = "feedaggregator/tag_detail.html"
    paginate_by = FEEDAGGREGATOR_PAGE_SIZE

    def get_context_data(self, **kwargs):
        context = super(TagDetail, self).get_context_data(**kwargs)
        context['tag'] = Tag.objects.get(slug=self.kwargs['tag'])
        return context

    def get_queryset(self):
        return Item.active.filter(tags__slug__in=[self.kwargs['tag']])


class TagCloud(TemplateView):
    template_name = "feedaggregator/tag_cloud.html"


def export_opml(request):
    """
    Generate an OPML file with the active feeds
    """
    feeds = Feed.active.all()
    site = get_current_site(request)
    return render(request, 'feedaggregator/export/opml.xml', {'feeds': feeds, 'site': site}, content_type="application/xhtml+xml")



# -*- coding: utf-8 -*-

"""
django-feedaggregator models

"""

from __future__ import unicode_literals
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.utils.encoding import python_2_unicode_compatible

from taggit.managers import TaggableManager
from uuslug import uuslug

from feedaggregator.managers import FeedManager, ItemManager


@python_2_unicode_compatible
class Feed(models.Model):
    """
    Represents a feed in RSS, or any other format.
    """
    title = models.CharField(_("title"), max_length=255, default="<Untitled>")
    feed_url = models.CharField(_("url"), unique=True, blank=False, max_length=500)
    site_url = models.URLField(_("site url"), blank=True)
    description = models.TextField(_("description"), blank=True)
    updated_on = models.DateTimeField(_("date published"), blank=True, null=True)
    slug = models.SlugField(unique=True)

    # Meta
    date_added = models.DateTimeField(_("added on"), auto_now_add=True)
    date_modified = models.DateTimeField(_("modified on"), auto_now=True)
    is_active = models.BooleanField(_("active"), blank=True, default=True)

    # Managers
    active = FeedManager()
    objects = models.Manager()

    class Meta:
        ordering = ['title']

    def save(self, *args, **kwargs):
        self.slug = uuslug(self.title, instance=self, max_length=50)
        super(Feed, self).save(*args, **kwargs)

    @property
    def is_new(self):
        return self.updated_on is None

    @property
    def has_items(self):
        return Item.active.filter(feed=self).exists()

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('feedaggregator:feed_detail', args=[self.slug])


@python_2_unicode_compatible
class Item(models.Model):
    """
    Represents an item on a Feed
    """
    feed = models.ForeignKey(Feed)
    title = models.CharField(_("title"), max_length=255, default="<Untitled>")
    link = models.URLField(_("url"), max_length=500)
    guid = models.CharField(_("global unique identifier"), max_length=500, blank=True, null=True)
    description = models.TextField(_("description"), blank=True)
    published_on = models.DateTimeField(_("date published"), blank=True, null=True)
    authors = models.ManyToManyField('Author', blank=True)
    tags = TaggableManager()
    slug = models.SlugField(unique=True)

    # Meta
    date_added = models.DateTimeField(_("added on"), auto_now_add=True)
    date_modified = models.DateTimeField(_("modified on"), auto_now=True)
    is_active = models.BooleanField(_("active"), blank=True, default=True)

    # Managers
    active = ItemManager()
    objects = models.Manager()

    class Meta:
        ordering = ['-published_on']

    def save(self, *args, **kwargs):
        self.slug = uuslug(self.title, instance=self, max_length=50)
        super(Item, self).save(*args, **kwargs)

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('feedaggregator:item_detail', kwargs={"slug": self.slug})


@python_2_unicode_compatible
class Author(models.Model):
    """
    Represents a writer of an item
    """
    name = models.CharField(_("name"), max_length=255, blank=True, default="<Unknown>")
    profile_url = models.URLField(_("profile"), blank=True)
    slug = models.SlugField(unique=True)

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        self.slug = uuslug(self.name, instance=self, max_length=50)
        super(Author, self).save(*args, **kwargs)



# -*- coding: utf-8 -*-
from django.contrib import admin

from feedaggregator.models import Feed, Item


class FeedAdmin(admin.ModelAdmin):
    list_display = ('__str__', 'updated_on', 'is_active')
    prepopulated_fields = {'slug': ('title',)}

admin.site.register(Feed, FeedAdmin)


class ItemAdmin(admin.ModelAdmin):
    list_display = ('title', 'feed', 'is_active', 'slug')

admin.site.register(Item, ItemAdmin)



# -*- coding: utf-8 -*-

from django.db import models


class FeedManager(models.Manager):
    def new_feeds(self):
        """
        Get the feeds that have never been fetched
        """
        return super(FeedManager, self).get_query_set().filter(is_new=True)

    def get_queryset(self):
        return super(FeedManager, self).get_queryset().filter(is_active=True)


class ItemManager(models.Manager):
    def get_queryset(self):
        return super(ItemManager, self).get_queryset().filter(is_active=True)



# -*- coding: utf-8 -*-
from celery import task
from time import mktime
from datetime import datetime
import logging

import feedparser

from feedaggregator.settings import FEEDAGGREGATOR_TAGS_LOWERCASE
from feedaggregator.models import Item

logger = logging.getLogger(__name__)


@task
def update_feed(feed):
    """
    Update the feed content and fetch latest items.
    """
    logger.debug("Processing feed %s" % feed)
    parsed_feed = feedparser.parse(feed.feed_url)

    if parsed_feed.bozo:  # Handle error in feed
        logger.error("Error in feed %s: %s" % (feed.feed_url, parsed_feed.bozo_exception))
        return

    if feed.is_new:
        title = parsed_feed.feed.get('title', "-")
        if not title:  # Check for empty strings
            title = "<Untitled>"

        link = parsed_feed.feed.get('link')
        description = parsed_feed.feed.get('description', "")

        if 'updated_parsed' in parsed_feed.feed:
            feed.updated_on = datetime.fromtimestamp(mktime(parsed_feed.feed.updated_parsed))

        # Save the feed info in the db
        feed.title = title
        feed.site_url = link
        feed.description = description
        feed.save()

    # Save the feed items
    for entry in parsed_feed.entries:
        if all(k in entry for k in ("title", "link")):
            print(entry)
            entry_title = entry.get('title', "<Untitled>")
            if not entry_title:  # Check for empty strings
                entry_title = "<Untitled>"

            # Sometimes feed titles change, this is why we don't user get_or_create
            try:
                feed_item = Item.objects.get(feed=feed, link=entry.link)
                feed_item.title = entry_title[:255]
            except Item.DoesNotExist:
                feed_item = Item(feed=feed, link=entry.link, title=entry_title[:255])

            feed_item.description = entry.get('description', "")
            feed_item.guid = entry.get('id', "")

            if 'published_parsed' in entry:
                feed_item.published_on = datetime.fromtimestamp(mktime(entry.published_parsed))
            feed_item.save()

            # Add the tags
            for tag_dict in entry.get('tags', ''):
                term = tag_dict.get('term', '')
                if FEEDAGGREGATOR_TAGS_LOWERCASE:
                    term = term.lower()
                feed_item.tags.add(term)



from django.contrib.sitemaps import Sitemap
from feedaggregator.models import Feed, Item


class ItemSitemap(Sitemap):
    changefreq = "monthly"
    priority = 0.8

    def items(self):
        return Item.active.all()

    def lastmod(self, obj):
        return obj.published_on


class FeedSitemap(Sitemap):
    changefreq = "monthly"
    priority = 0.5

    def items(self):
        return Feed.active.all()

    def lastmod(self, obj):
        return obj.updated_on



# -*- coding: utf-8 -*-
import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-feedaggregator',
    version='0.2',
    packages=['feedaggregator'],
    include_package_data=True,
    description='Django app to create an RSS feeds aggregator',
    long_description=README,
    url='https://github.com/fasouto/feedaggregator/',
    install_requires=[
        'feedparser',
        'listparser',
        'django-uuslug',
        'django-taggit',
        'celery',
        'django-taggit-templatetags2'
    ],
    author='Fabio Souto',
    author_email='fabio@fabiosouto.me',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content :: News/Diary',
    ],
)



from django.contrib.syndication.views import Feed as DjangoFeed
from django.core.urlresolvers import reverse
from django.utils.feedgenerator import Atom1Feed

from feedaggregator.models import Item, Feed
from feedaggregator.settings import FEEDAGGREGATOR_TITLE, FEEDAGGREGATOR_RSS_SIZE, FEEDAGGREGATOR_RSS_DESCRIPTION


class LatestItemsFeed(DjangoFeed):
    title = FEEDAGGREGATOR_TITLE
    description = FEEDAGGREGATOR_RSS_DESCRIPTION

    def items(self):
        return Item.active.order_by('-date_modified')[:FEEDAGGREGATOR_RSS_SIZE]

    def link(self):
        return reverse("feedaggregator:rss_latest_items")

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.description

    def item_published(self, item):
        return item.published_on


class AtomLatestItemsFeed(LatestItemsFeed):
    feed_type = Atom1Feed
    subtitle = LatestItemsFeed.description
